/*
 *  802.11 40/104 bit WEP Key Cracker
 *
 *  Copyright (C) 2004  Christophe Devine
 *
 *  Advanced WEP attacks developed by KoreK
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/ioctl.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <math.h>

#include "pcap.h"

#define AMOD(x,y) ((x) % (y) < 0 ? ((x) % (y)) + (y) : (x) % (y))
#define SWAP(x,y) { unsigned char tmp = x; x = y; y = tmp; }

#define SUCCESS 0
#define FAILURE 1

char usage[] =

"\n"
"  aircrack 1.2 - (C) 2004 Christophe Devine\n"
"\n"
"  usage: %s [options] <pcap filename(s)>\n"
"\n"
"      -b <bssid> : identifier of the network to crack\n"
"      -d <start> : debug - specify beginning of the key\n"
"      -f <fudge> : bruteforce fudge factor (default: 2)\n"
"      -k <keyid> : WEP KeyID to crack (0-3, default: 0)\n"
"      -n <nbits> : WEP key length (40/104, default: 40)\n"
"      -p <nfork> : SMP support: # of processes to start\n"
"\n";

/* command-line parameters */

int bssid_set = 0;              /* check flag   */
int debug_lvl = 0;              /* debug level  */
unsigned char bssid[6];         /* AP's BSSID   */
unsigned char debug[13];        /* partial key  */
int keyid  = 0;                 /* WEP KeyID    */
int weplen = 5;                 /* WEP key len. */
int ffact  = 2;                 /* fudge factor */
int nfork  = 1;                 /* nb. of forks */

/* runtime global data */

unsigned char buffer[65536];    /* buffer for reading packets   */
unsigned char kcheck[8][10];    /* 8 packets to verify wepkey   */
unsigned char wepkey[13];       /* the current chosen WEP key   */
unsigned char *ivbuf;           /* buffer for the unique IVs    */
unsigned long nb_ivs;           /* number of elements in ivbuf  */
unsigned long tried;            /* total # of keys tried so far */
time_t tm_start, tm_prev;       /* for displaying elapsed time  */
char *test_unique_ivs;          /* to avoid adding duplicates   */
int mc_pipe[256][2];            /* master->child control pipe   */ 
int cm_pipe[256][2];            /* child->master results pipe   */
int weaks[13];                  /* nb of weak ivs for each byte */
int fudge[13];                  /* bruteforce level (1 to 256)  */
int depth[13];                  /* how deep we are in the fudge */

struct byte_stat
{
    int index;
    int votes;
}
wpoll[13][256];                 /* FMS + Korek attack: stats.   */

int read_ivs( char *filename, int *nb_chk );
int calc_votes( int child );
int do_wep_crack( int B );

/* routine that handles signals */

void my_sighandler( int signum )
{
    if( signum == SIGTERM )
        exit( 0 );

    if( signum == SIGCHLD )
    {
        printf( "got unexpected SIGCHLD, exiting.\n" );
        kill( 0, SIGTERM );
        exit( FAILURE );
    }
}

int main( int argc, char *argv[] )
{
    char *s;
    int i, n, nb_chk, pid;

    /* check the arguments */

    if( argc < 2 )
    {
    usage:
        printf( usage, argv[0] );
        return( FAILURE );
    }

    while( 1 )
    {
        int option = getopt( argc, argv, "b:d:f:k:n:p:" );

        if( option < 0 ) break;

        switch( option )
        {
            case 'b':

                i = 0;
                s = optarg;

                while( sscanf( s, "%x", &n ) == 1 )
                {
                    if( n < 0 || n > 255 )
                        goto usage;

                    bssid[i] = n;

                    if( ++i == 6 ) break;

                    if( ! ( s = strchr( s, ':' ) ) )
                        break;

                    s++;
                }

                if( i != 6 ) goto usage;
                bssid_set = 1;
                break;

            case 'd' :

                i = 0;
                s = optarg;

                while( sscanf( s, "%x", &n ) == 1 )
                {
                    if( n < 0 || n > 255 )
                        goto usage;

                    debug[i] = n;

                    if( ++i == 13 ) break;

                    if( ! ( s = strchr( s, ':' ) ) )
                        break;

                    s++;
                }

                if( i == 0 ) goto usage;
                debug_lvl = i;
                break;

            case 'f' :

                if( sscanf( optarg, "%d", &ffact ) != 1 )
                    goto usage;

                if( ffact < 1 )
                    goto usage;

                break;

            case 'k' :

                if( sscanf( optarg, "%d", &keyid ) != 1 )
                    goto usage;

                if( keyid < 0 || keyid > 3 )
                    goto usage;

                break;

            case 'n' :

                if( sscanf( optarg, "%d", &weplen ) != 1 )
                    goto usage;

                if( weplen != 40 && weplen != 104 )
                    goto usage;

                weplen /= 8;

                break;

            case 'p' :

                if( sscanf( optarg, "%d", &nfork ) != 1 )
                    goto usage;

                if( nfork < 1 || nfork > 256 )
                    goto usage;

                break;

            default : goto usage;
        }
    }

    if( ! ( argc - optind ) )
        goto usage;

    /* initialize all the data */

    nb_ivs = 0;
    nb_chk = 0;

    if( ! ( ivbuf = (unsigned char *)
                        malloc( 5 * 256 * 256 * 256 ) ) )
    {
        perror( "malloc" );
        return( FAILURE );
    }

    if( ! ( test_unique_ivs = (char *)
                                malloc( 256 * 256 * 256 ) ) )
    {
        perror( "malloc" );
        return( FAILURE );
    }

    memset( test_unique_ivs, 0, 256 * 256 * 256 );

    /* read the packets from each file */

    while( optind != argc )
    {
        if( read_ivs( argv[optind], &nb_chk ) != SUCCESS )
            return( FAILURE );

        optind++;
    }

    free( test_unique_ivs );

    if( nb_chk < 8 )
    {
        fprintf( stderr, "Could not get enought check packets!\n" );
        return( FAILURE );
    }

    /* fork the childs */

    signal( SIGTERM, SIG_IGN );
    signal( SIGCHLD, my_sighandler );

    for( i = 0; i < nfork; i++ )
    {
        pipe( mc_pipe[i] );
        pipe( cm_pipe[i] );

        if( ( pid = fork() ) < 0 )
        {
            perror( "fork" );
            return( FAILURE );
        }

        if( ! pid )
        {
            signal( SIGTERM, my_sighandler );
            return( calc_votes( i ) );
        }
    }

    /* launch the attack */

    tm_start = tm_prev = time( NULL );

    printf( "\33[2J\33[2;1H" );
    fflush( stdout );

    return( do_wep_crack( 0 ) );
}

int read_ivs( char *filename, int *nb_chk )
{
    int i, n, infile;
    unsigned long cnt1;
    unsigned long cnt2;
    unsigned char *h80211;
    struct pcap_pkthdr pkh;
    struct pcap_file_header pfh;

    /* open the file and check the pcap header */

    printf( "Opening pcap file %s\n", filename );

    if( ( infile = open( filename, O_RDONLY | O_LARGEFILE ) ) < 0 )
    {
        perror( "open" );
        return( FAILURE );
    }

    n = sizeof( struct pcap_file_header );

    if( read( infile, &pfh, n ) != n )
    {
        perror( "read" );
        return( FAILURE );
    }

    if( pfh.magic != TCPDUMP_MAGIC )
    {
        fprintf( stderr, "wrong magic from pcap file header\n"
                         "(got 0x%08X, expected 0x%08X)\n",
                         pfh.magic, TCPDUMP_MAGIC );
        return( FAILURE );
    }

    if( pfh.linktype != LINKTYPE_IEEE802_11 &&
        pfh.linktype != LINKTYPE_PRISM_HEADER )
    {
        fprintf( stderr, "unsupported pcap header linktype %d\n",
                 pfh.linktype );
        return( FAILURE );
    }

    tm_prev = time( NULL );

    cnt1 = cnt2 = 0;

    while( 1 )
    {
        /* every 1s, update the display */

        if( time( NULL ) - tm_prev >= 1 )
        {
            tm_prev = time( NULL );
            printf( "\r\33[2KReading packets: total = %ld"
                    ", usable = %ld\r", cnt1, cnt2 );
            fflush( stdout );
        }

        /* read one packet */

        n = sizeof( pkh );

        if( read( infile, &pkh, n ) != n )
            break;

        n = pkh.len;

        if( read( infile, buffer, n ) != n )
            break;

        cnt1++;

        h80211 = buffer;

        if( pfh.linktype == LINKTYPE_PRISM_HEADER )
        {
            /* remove the prism header if necessary */

            n = *(int *)( h80211 + 4 );
            h80211 += n; pkh.len -= n;

            if( pkh.len <= 0 )
            {
                fprintf( stderr, "\nFIXME: wrong prism header size ?\n" );
                continue;
            }
        }

        /* minimum encrypted data packet size: 40 = 24  *
         * (802.11) + 4 (IV, KeyID) + 8 (LLC) + 4 (FCS) */

        if( pkh.len < 40 ) continue;

        /* is it an encrypted data packet ? */

        if( ( h80211[0] & 0x0C ) != 0x08 ) continue;
        if( ( h80211[1] & 0x40 ) != 0x40 ) continue;

        /* is it the correct keyid ? */

        if( ( h80211[27] >> 6 ) != keyid ) continue;

        /* check if the bssids match */

        switch( h80211[1] & 3 )
        {
            case  0: i = 16; break;      /* DA, SA, BSSID  */
            case  1: i =  4; break;      /* BSSID, SA, DA  */
            case  2: i = 10; break;      /* DA, BSSID, SA  */
            default: i =  4; break;      /* RA, TA, DA, SA */
        }

        if( ! bssid_set )
        {
            memcpy( bssid, h80211 + i, 6 );

            printf( "\33[2KChoosing first encrypted BSSID"
                    " = %02X:%02X:%02X:%02X:%02X:%02X\n",
                    bssid[0], bssid[1], bssid[2],
                    bssid[3], bssid[4], bssid[5] );

            bssid_set = 1;
        }
        else
            if( memcmp( bssid, h80211 + i, 6 ) )
                continue;

        /* have we already seen this IV ? */

        n = ( h80211[24] << 16 )
          | ( h80211[25] <<  8 )
          | ( h80211[26]       );

        if( test_unique_ivs[n] ) continue;

        test_unique_ivs[n] = 1;

        if( *nb_chk < 8 )
        {
            /* this packet's first bytes will be used to check *
             * if the current chosen WEP key is the valid one  */

            memcpy( kcheck[*nb_chk], h80211 + 24, 10 );

            kcheck[*nb_chk][4] ^= 0xAA;
            kcheck[*nb_chk][5] ^= 0xAA;
            kcheck[*nb_chk][6] ^= 0x03;
/*
            kcheck[*nb_chk][7] ^= 0x00;
            kcheck[*nb_chk][8] ^= 0x00;
            kcheck[*nb_chk][9] ^= 0x00;
*/
            (*nb_chk)++;
        }

        /* might be a weak IV for keybyte B, so add it */

        ivbuf[nb_ivs * 5    ] = h80211[24];
        ivbuf[nb_ivs * 5 + 1] = h80211[25];
        ivbuf[nb_ivs * 5 + 2] = h80211[26];

        /* also add the first two bytes of RC4's output */

        ivbuf[nb_ivs * 5 + 3] = h80211[28] ^ 0xAA;
        ivbuf[nb_ivs * 5 + 4] = h80211[29] ^ 0xAA;

        cnt2++; nb_ivs++;
    }

    printf( "\r\33[2KReading packets: total = %ld"
            ", usable = %ld.\n", cnt1, cnt2 );

    return( SUCCESS );
}

/* this routine sets up the key schedule using wepkey and *
 * each 8 packets IV, then checks if RC4's output matches */

int check_wepkey( void )
{
    unsigned char S[256], K[16];
    int ntest, i, j, d, dmask;

    dmask = 3 + weplen - 1;

    memcpy( K + 3, wepkey, weplen );

    for( ntest = 0; ntest < 8; ntest++ )
    {
        for( i = 0; i < 256; i++ )
            S[i] = i;

        memcpy( K, kcheck[ntest], 3 );

        for( i = j = d = 0; i < 256; i++ )
        {
            j = ( j + S[i] + K[d]) & 0xFF;
            SWAP( S[i], S[j] );
            d++; d &= dmask;
        }

        for( i = 1, j = 0; i < 7; i++ )
        {
            j = ( j + S[i] ) & 0xFF;
            SWAP(S[i], S[j]);

            if( S[(S[i] + S[j]) & 0xFF] !=
                kcheck[ntest][3 + i] )
                return( FAILURE );
        }
    }

    return( SUCCESS );
}

/* this routine displays a bunch of statistical   *
 * data about the current state of the FMS attack */

void show_stats( int B )
{
    time_t delta;
    struct winsize ws;
    int i, et_h, et_m, et_s;

    tm_prev = time( NULL );
    delta = tm_prev - tm_start;

    if( ioctl( 0, TIOCGWINSZ, &ws ) < 0 )
    {
        ws.ws_row = 25;
        ws.ws_col = 80;
    }

    if( ! delta ) delta++;

    et_h =   delta / 3600;
    et_m = ( delta - et_h * 3600 ) / 60;
    et_s =   delta - et_h * 3600 - et_m * 60;

    printf( "\33[2;%dH\33[1maircrack 1.2\33[0m\n\n",
            ( ws.ws_col - 12 ) / 2 ); 

    printf( "   * Got %8ld unique IVs | fudge factor = %d\n",
            nb_ivs, ffact );

    printf( "   * Elapsed time [%02d:%02d:%02d] | tried "
            "%ld keys at %ld k/m\n", et_h, et_m, et_s, tried,
            ( 60 * tried ) / delta );

    printf( "\n   KB  weaks   depth   votes\n" );

    for( i = 0; i <= B; i++ )
    {
        int j, k = ( ws.ws_col - 24 ) / 9;

        printf( "   %2d %5d  %3d/%3d   ",
                i, weaks[i], depth[i], fudge[i] );

        for( j = depth[i]; j < k + depth[i]; j++ )
        {
            if( j >= 256 ) break;

            printf( "%02X(%4d) ", wpoll[i][j].index,
                                  wpoll[i][j].votes );
        }

        printf( "\n" );
    }

    printf( "\33[J\n" );

    fflush( stdout );
}

/* each child computes the votes over nb_ivs / nforks */

int calc_votes( int child )
{
    unsigned long xv, min, max;
    unsigned char R[256], jj[256];
    unsigned char S[256], Si[256];
    unsigned char K[16];

    unsigned char io1, o1, io2, o2;
    unsigned char Sq, dq, Kq, jq, q;
    unsigned char S1, S2;

    int i, j, B;
    int votes[256];
    int eject[256];

    signal( SIGTERM, my_sighandler );

wait_for_master:

    if( read( mc_pipe[child][0], &B, sizeof( int ) ) !=
                                     sizeof( int ) )
    {
        if( errno == EINTR ) goto wait_for_master;
        perror( "in calc_votes: read()" );
        return( FAILURE );
    }

retry_mc_read_2:

    if( read( mc_pipe[child][0], K + 3, 13 ) != 13 )
    {
        if( errno == EINTR ) goto retry_mc_read_2;
        perror( "in calc_votes: read()" );
        return( FAILURE );
    }

    for( i = 0; i < 256; i++ )
    {
        votes[i] = 0;
        eject[i] = 0;
        R[i] = i;
    }

    q = 3 + B;

    min = 5 * ( ( (     child ) * nb_ivs ) / nfork );
    max = 5 * ( ( ( 1 + child ) * nb_ivs ) / nfork );

    /*
     *                        JABBERWOCKY
     */

    for( xv = min; xv < max; xv += 5 )
    {
        memcpy( K, &ivbuf[xv], 3 );
        memcpy( S,  R, 256 );
        memcpy( Si, R, 256 );

        /*
         *      `Twas brillig, and the slithy toves
         *        Did gyre and gimble in the wabe:
         *         All mimsy were the borogoves,
         *          And the mome raths outgrabe.
         */

        if( weplen == 13 )
        {
            for( i = j = 0; i < q; i++ )
            {
                jj[i] = j = ( j + S[i] + K[i & 15] ) & 0xFF;
                SWAP( S[i], S[j] );
            }
        }

        if( weplen == 5 )
        {
            for( i = j = 0; i < q; i++ )
            {
                jj[i] = j = ( j + S[i] + K[i & 7] ) & 0xFF;
                SWAP( S[i], S[j] );
            }
        }

        /*
         *      Beware the Jabberwock, my son!
         *        The jaws that bite, the claws that catch!
         *      Beware the Jubjub bird, and shun
         *        The frumious Bandersnatch!
         */

        i = q; do { i--; SWAP(Si[i],Si[jj[i]]); } while( i != 0 );

        o1 = ivbuf[xv + 3]; io1 = Si[o1]; S1 = S[1];
        o2 = ivbuf[xv + 4]; io2 = Si[o2]; S2 = S[2];
        Sq = S[q]; dq = Sq + jj[q - 1];

        if( S2 == 0 )
        {
            if( ( S1 == 2 ) && ( o1 == 2 ) )
            {
                Kq = 1 - dq; eject[Kq]++;
                Kq = 2 - dq; eject[Kq]++;
            }
            else if( o2 == 0 )
            {
                Kq = 2 - dq; eject[Kq]++;
            }
        }
        else
        {
            if( ( o2 == 0 ) && ( Sq == 0 ) )
            {
                Kq = 2 - dq; votes[Kq] += 14;
            }
        }

        /*
         *      He took his vorpal sword in hand:
         *        Long time the manxome foe he sought --
         *      So rested he by the Tumtum tree,
         *        And stood awhile in thought.
         */

        if( ( S1 == 1 ) && ( o1 == S2 ) )
        {
            Kq = 1 - dq; eject[Kq]++;
            Kq = 2 - dq; eject[Kq]++;
        }

        if( ( S1 == 0 ) && ( S[0] == 1 ) && ( o1 == 1 ) )
        {
            Kq = 0 - dq; eject[Kq]++;
            Kq = 1 - dq; eject[Kq]++;
        }

        /*
         *      And, as in uffish thought he stood,
         *        The Jabberwock, with eyes of flame,
         *      Came whiffling through the tulgey wood,
         *        And burbled as it came!
         */

        if( S1 == q )
        {
            if( o1 == q )
            {
                Kq = Si[0] - dq; votes[Kq] += 16;
            }
            else if( ( ( 1 - q - o1 ) & 0xFF ) == 0 )
            {
                Kq =   io1 - dq; votes[Kq] += 15;
            }
            else if( io1 < q )
            {
                jq = Si[( io1 - q ) & 0xFF];

                if( jq != 1 )
                {
                    Kq = jq - dq; votes[Kq] += 10;
                }
            }
        }

        /*
         *      One, two! One, two! And through and through
         *        The vorpal blade went snicker-snack!
         *      He left it dead, and with its head
         *        He went galumphing back.
         */

        if( ( io1 == 2 ) && ( S[q] == 1 ) )
        {
            Kq = 1 - dq; votes[Kq] += 10;
        }

        if( S[q] == q )
        {
            if( ( S1 == 0 ) && ( o1 == q ) )
            {
                Kq = 1 - dq; votes[Kq] += 15;
            }
            else if( ( ( ( 1 - q - S1 ) & 0xFF ) == 0 ) &&
                     ( o1 == S1 ) )
            {
                Kq = 1 - dq; votes[Kq] += 15;
            }
            else if( ( S1 >= ( ( -q ) & 0xFF ) ) &&
                     ( ( ( q + S1 - io1 ) & 0xFF ) == 0 ) )
            {
                Kq = 1 - dq; votes[Kq] += 10;
            }
        }

        if( ( S1 < q ) && ( ( ( S1 + S[S1] - q ) & 0xFF ) == 0 ) &&
            ( io1 != 1 ) && ( io1 != S[S1] ) )
        {
            Kq = io1 - dq; votes[Kq] += 11;
        }

        /*
         *      And, has thou slain the Jabberwock?
         *        Come to my arms, my beamish boy!
         *      O frabjous day! Callooh! Callay!'
         *        He chortled in his joy.
         */

        if( ( S1 > q ) && ( ( ( S2 + S1 - q ) & 0xFF ) == 0 ) )
        {
            if( o2 == S1 )
            {
                jq = Si[S1 - S2];

                if( ( jq != 1 ) && ( jq != 2 ) )
                {
                    Kq = jq - dq; votes[Kq] += 11;
                }
            }
            else if( o2 == ( 2 - S2 ) )
            {
                jq = io2;

                if( ( jq != 1 ) && ( jq != 2 ) )
                {
                    Kq = jq - dq; votes[Kq] += 11;
                }
            }
        }

        /*
         *      `Twas brillig, and the slithy toves
         *        Did gyre and gimble in the wabe:
         *         All mimsy were the borogoves,
         *          And the mome raths outgrabe.
         */

        if( S1 == 2 )
        {
            if( q == 4 )
            {
                if( o2 == 0 )
                {
                    Kq = Si[0] - dq; votes[Kq] += 16;
                }
                else
                {
                    if( ( jj[1] == 2 ) && ( io2 == 0 ) )
                    {
                        Kq = Si[254] - dq; votes[Kq] += 10;
                    }

                    if( ( jj[1] == 2 ) && ( io2 == 2 ) )
                    {
                        Kq = Si[255] - dq; votes[Kq] += 10;
                    }
                }
            }
            else if( ( q > 4U ) && ( ( S[4] + 2 ) == q ) &&
                     ( io2 != 1 ) && ( io2 != 4 ) )
            {
                Kq = io2 - dq; votes[Kq] += 10;
            }
        }
    }

retry_cm_write_1:

    if( write( cm_pipe[child][1], votes, sizeof( votes ) ) !=
                                         sizeof( votes ) )
    {
        if( errno == EINTR ) goto retry_cm_write_1;
        perror( "in calc_votes: write()" );
        return( FAILURE );
    }

retry_cm_write_2:

    if( write( cm_pipe[child][1], eject, sizeof( eject ) ) !=
                                         sizeof( eject ) )
    {
        if( errno == EINTR ) goto retry_cm_write_2;
        perror( "in calc_votes: write()" );
        return( FAILURE );
    }

    goto wait_for_master;
}

/* routine used to sort the votes */

int cmp_votes( const void *bs1, const void *bs2 )
{
    if( ((struct byte_stat *) bs1)->votes <
        ((struct byte_stat *) bs2)->votes )
        return(  1 );

    if( ((struct byte_stat *) bs1)->votes >
        ((struct byte_stat *) bs2)->votes )
        return( -1 );

    return( SUCCESS );
}

/* this recursive routine performs the FMS attack itself */

int do_wep_crack( int B )
{
    int child, i;
    int votes[256];
    int eject[2][256];

    for( i = 0; i < 256; i++ )
    {
        wpoll[B][i].index = i;
        wpoll[B][i].votes = 0;
        eject[0][i] = 0;
    }

    /* send B and wepkey to each child */

    for( child = 0; child < nfork; child++ )
    {
    retry_mc_write_1:

        if( write( mc_pipe[child][1], &B, sizeof( int ) ) !=
                                          sizeof( int ) )
        {
            if( errno == EINTR ) goto retry_mc_write_1;
            perror( "in do_wep_crack: write()" );
            return( FAILURE );
        }

    retry_mc_write_2:

        if( write( mc_pipe[child][1], wepkey, 13 ) != 13 )
        {
            if( errno == EINTR ) goto retry_mc_write_2;
            perror( "in do_wep_crack: write()" );
            return( FAILURE );
        }
    }

    /* collect the poll results from each child */

    for( child = 0; child < nfork; child++ )
    {
    retry_cm_read_1:

        if( read( cm_pipe[child][0], &votes, sizeof( votes ) ) !=
                                             sizeof( votes ) )
        {
            if( errno == EINTR ) goto retry_cm_read_1;
            perror( "in do_wep_crack: read()" );
            return( FAILURE );
        }

    retry_cm_read_2:

        if( read( cm_pipe[child][0], &eject[1], sizeof( eject[1] ) ) !=
                                                sizeof( eject[1] ) )
        {
            if( errno == EINTR ) goto retry_cm_read_2;
            perror( "in do_wep_crack: read()" );
            return( FAILURE );
        }

        for( i = 0; i < 256; i++ )
        {
            wpoll[B][i].votes += votes[i];
            eject[0][i] += eject[1][i];
        }
    }

    /* reject the "impossible" keybytes */

    for( i = 0; i < 256; i++ )
    {
        switch( eject[0][i] )
        {
            case  0: break;
            case  1: wpoll[B][i].votes -= 10; break;
            case  2: wpoll[B][i].votes -= 30; break;
            case  3: wpoll[B][i].votes -= 60; break;
            default: wpoll[B][i].votes  = -1; break;
        }
    }

    /* compute the average number of weak IVs */

    weaks[B] = 0;

    for( i = 0; i < 256; i++ )
        if( wpoll[B][i].votes > 0 )
            weaks[B] += wpoll[B][i].votes;

    weaks[B] /= 10;

    /* set votes to the max if keybyte is user-defined */

    if( B < debug_lvl )
        wpoll[B][debug[B]].votes = 999;

    /* sort the votes, highest ones first */

    qsort( wpoll[B], 256, sizeof( struct byte_stat ), cmp_votes );

    /* see how far we should go based on the number of weaks ivs */

    for( fudge[B] = 1; fudge[B] < 256; fudge[B]++ )
        if( wpoll[B][fudge[B]].votes < wpoll[B][0].votes / ffact )
            break;

    /* try the most likely n votes, where n is our current fudge */ 

    for( depth[B] = 0; depth[B] < fudge[B]; depth[B]++ )
    {
        wepkey[B] = wpoll[B][depth[B]].index;

        show_stats( B );

        if( B < weplen - 1 )
        {
            /* this keybyte has been set, attack the next one */

            if( ! do_wep_crack( B + 1 ) )
                return( SUCCESS );
        }
        else
        {
            /* last keybyte reached, so check if wepkey is valid */

            tried++;

            if( ! check_wepkey() )
            {
                /* we have a valid key */

                show_stats( B );

                printf( "\33[%dC\33[31;1mKEY FOUND! [",
                        28 - weplen );

                for( i = 0; i < weplen; i++ )
                    printf( "%02X%c", wepkey[i],
                            ( i + 1 < weplen ) ? ':' : ']' );

                printf( "\33[0m\n\n" );

                kill( 0, SIGTERM );
                return( SUCCESS );
            }
        }
    }

    if( B == 0 )
    {
        printf( "   No luck, sorry.\n\n" );
        kill( 0, SIGTERM );
    }

    return( FAILURE );
}
